@@ -1,13 +1,11 @@
 /***************************************************************************
- $RCSfile: account_p.h,v $
- -------------------
- cvs         : $Id: account_p.h,v 1.6 2005/04/22 05:13:22 aquamaniac Exp $
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -18,25 +16,43 @@
 #include "account_l.h"
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/inherit.h>
-#include <gwenhywfar/misc2.h>
+#include <gwenhywfar/list2.h>
 #include <gwenhywfar/db.h>
 
 
 struct AB_ACCOUNT {
   GWEN_INHERIT_ELEMENT(AB_ACCOUNT)
   GWEN_LIST_ELEMENT(AB_ACCOUNT)
-  GWEN_TYPE_UINT32 usage;
-  GWEN_TYPE_UINT32 uniqueId;
+  uint32_t usage;
+
+  uint32_t uniqueId;
   AB_ACCOUNT_TYPE accountType;
 
   AB_BANKING *banking;
   AB_PROVIDER *provider;
-  char *providerName;
-  GWEN_DB_NODE *data;
-  int availability;
+  char *backendName;
+
+  char *subAccountId;
+  char *accountNumber;
+  char *bankCode;
+  char *accountName;
+  char *bankName;
+  char *iban;
+  char *bic;
+  char *ownerName;
+  char *currency;
+  char *country;
+
+  GWEN_STRINGLIST *userIds;
+  GWEN_STRINGLIST *selectedUserIds;
+
+  GWEN_DB_NODE *appData;
+  GWEN_DB_NODE *providerData;
+
+  char *dbId;
 };
 
-AB_ACCOUNT *AB_Account__freeAll_cb(AB_ACCOUNT *a, void *userData);
+static AB_ACCOUNT *AB_Account__freeAll_cb(AB_ACCOUNT *a, void *userData);
 
 
 
