@@ -1,13 +1,14 @@
 /***************************************************************************
- $RCSfile: country.c,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: country.c,v 1.5 2005/08/01 12:17:22 cstim Exp $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 #ifdef HAVE_CONFIG_H
@@ -77,7 +78,7 @@ static AB_COUNTRY ab_country_list[]= {
 { I18N_NOOP("Djibouti"), "DJ", 262, I18N_NOOP("Djibouti Franc"),  "DJF" },
 { I18N_NOOP("Dominica"), "DM", 212, I18N_NOOP("East Caribbean Dollar"),  "XCD" },
 { I18N_NOOP("Dominican Republic"), "DO", 214, I18N_NOOP("Dominican Republic Peso"),  "DOP" },
-{ I18N_NOOP("Ecuador"), "EC", 218, I18N_NOOP("US&nbsp;Dollar "),  "USD" },
+{ I18N_NOOP("Ecuador"), "EC", 218, I18N_NOOP("US Dollar"),  "USD" },
 { I18N_NOOP("Egypt"), "EG", 818, I18N_NOOP("Egytian Pound"),  "EGP" },
 { I18N_NOOP("El Salvador"), "SV", 222, I18N_NOOP("El Salvadorian Col&oacute;n"),  "SVC" },
 { I18N_NOOP("Equatorial Guinea"), "GQ", 226, I18N_NOOP("Franc de la Communaut&eacute; financi&egrave;re africaine and Ekwele"),  "XAF" },
@@ -170,7 +171,7 @@ static AB_COUNTRY ab_country_list[]= {
 { I18N_NOOP("Northern Mariana Islands"), "MP", 580, I18N_NOOP("US Dollar"),  "USD" },
 { I18N_NOOP("Norway"), "NO", 578, I18N_NOOP("Norwegian Krone"),  "NOK" },
 { I18N_NOOP("Palau"), "PW", 585, I18N_NOOP("US Dollar"),  "USD" },
-{ I18N_NOOP("Panama"), "PA", 591, I18N_NOOP("Balboa and US&nbsp;Dollar"),  "PAB" },
+{ I18N_NOOP("Panama"), "PA", 591, I18N_NOOP("Balboa and US Dollar"),  "PAB" },
 { I18N_NOOP("Papua New Guinea"), "PG", 598, I18N_NOOP("Kina"),  "PGK" },
 { I18N_NOOP("Paraguay"), "PY", 600, I18N_NOOP("Guarani"),  "PYG" },
 { I18N_NOOP("Peru"), "PE", 604, I18N_NOOP("Inti and New Sol "),  "PEI" },
@@ -231,13 +232,17 @@ static AB_COUNTRY ab_country_list[]= {
 { I18N_NOOP("Uzbekistan"), "UZ", 860, I18N_NOOP("Uzbekistani Som "),  "UZS" },
 { I18N_NOOP("Venezuela"), "VE", 862, I18N_NOOP("Bolivar"),  "VEB" },
 { I18N_NOOP("Viet Nam"), "VN", 704, I18N_NOOP("D&ocirc;ng"),  "VND" },
-{ I18N_NOOP("Virgin Islands "), "VG", 92, I18N_NOOP("US Dollar "),  "USD" },
+{ I18N_NOOP("Virgin Islands "), "VG", 92, I18N_NOOP("US Dollar"),  "USD" },
 { I18N_NOOP("Virgin Islands "), "VI", 850, I18N_NOOP("US Dollar"),  "USD" },
 { I18N_NOOP("West Africa"), "XO", 0, I18N_NOOP("West African Franc"),  "XOF" },
 { I18N_NOOP("Western Sahara"), "EH", 732, I18N_NOOP("Moroccan Dirham and Mauritanian Ouguiya"),  "MAD" },
 { I18N_NOOP("Yugoslavia"), "YU", 0, I18N_NOOP("New Dinar"),  "YUD" },
 { I18N_NOOP("Zambia"), "ZM", 894, I18N_NOOP("Zambian Kwacha"),  "ZMK" },
-{ I18N_NOOP("Zimbabwe"), "ZW", 716, I18N_NOOP("Zimbabwe Dollar"),  "ZWD" }
+{ I18N_NOOP("Zimbabwe"), "ZW", 716, I18N_NOOP("Zimbabwe Dollar"),  "ZWD" },
+  /* Watch out: The last element *has* to be this all-zero
+     element, or else the iterations will not find the end of this
+     list. */
+{ 0, 0, 0, 0, 0 }
 };
 
 
