@@ -1,13 +1,11 @@
 /***************************************************************************
- $RCSfile: jobgetstandingorders_p.h,v $
- -------------------
- cvs         : $Id: jobgetstandingorders_p.h,v 1.1 2005/05/01 20:14:12 aquamaniac Exp $
  begin       : Mon Mar 01 2004
- copyright   : (C) 2004 by Martin Preuss
+ copyright   : (C) 2004-2013 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -35,7 +33,7 @@ struct AB_JOB_GETSTANDINGORDERS {
 
 };
 
-void AB_JobGetStandingOrders_FreeData(void *bp, void *p);
+static void GWENHYWFAR_CB AB_JobGetStandingOrders_FreeData(void *bp, void *p);
 
 
 
