@@ -1,13 +1,11 @@
 /***************************************************************************
- $RCSfile: provider_p.h,v $
- -------------------
- cvs         : $Id: provider_p.h,v 1.11 2005/01/22 17:17:00 aquamaniac Exp $
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -18,7 +16,7 @@
 #include <aqbanking/banking.h>
 #include <gwenhywfar/misc.h>
 #include <gwenhywfar/inherit.h>
-#include <gwenhywfar/misc2.h>
+#include <gwenhywfar/list2.h>
 #include <gwenhywfar/plugin.h>
 
 
@@ -37,16 +35,36 @@ struct AB_PROVIDER {
   AB_PROVIDER_ADDJOB_FN addJobFn;
   AB_PROVIDER_EXECUTE_FN executeFn;
   AB_PROVIDER_RESETQUEUE_FN resetQueueFn;
-  AB_PROVIDER_GETACCOUNTLIST_FN getAccountListFn;
-  AB_PROVIDER_UPDATEACCOUNT_FN updateAccountFn;
-  AB_PROVIDER_ADDACCOUNT_FN addAccountFn;
+
+  AB_PROVIDER_EXTEND_USER_FN extendUserFn;
+  AB_PROVIDER_EXTEND_ACCOUNT_FN extendAccountFn;
+
+  AB_PROVIDER_UPDATE_FN updateFn;
+
+  AB_PROVIDER_GET_NEWUSER_DIALOG_FN getNewUserDialogFn;
+  AB_PROVIDER_GET_EDITUSER_DIALOG_FN getEditUserDialogFn;
+
+  AB_PROVIDER_GET_NEWACCOUNT_DIALOG_FN getNewAccountDialogFn;
+  AB_PROVIDER_GET_EDITACCOUNT_DIALOG_FN getEditAccountDialogFn;
+
+  AB_PROVIDER_GET_USERTYPE_DIALOG_FN getUserTypeDialogFn;
+
   GWEN_PLUGIN *plugin;
 
-  GWEN_TYPE_UINT32 usage;
-  GWEN_TYPE_UINT32 flags;
+  uint32_t usage;
+  uint32_t flags;
   int isInit;
 };
 
 
 
+typedef struct AB_PLUGIN_PROVIDER AB_PLUGIN_PROVIDER;
+struct AB_PLUGIN_PROVIDER {
+  AB_PLUGIN_PROVIDER_FACTORY_FN pluginFactoryFn;
+};
+
+static void GWENHYWFAR_CB AB_Plugin_Provider_FreeData(void *bp, void *p);
+
+
+
 #endif /* AQBANKING_PROVIDER_P_H */
