@@ -1,13 +1,14 @@
 /***************************************************************************
- $RCSfile: accstatus_l.h,v $
+ $RCSfile$
                              -------------------
-    cvs         : $Id: accstatus_l.h,v 1.2 2005/05/13 14:42:37 aquamaniac Exp $
+    cvs         : $Id$
     begin       : Mon Apr 05 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
@@ -17,9 +18,7 @@
 #include <aqbanking/accstatus.h>
 #include <gwenhywfar/misc.h>
 
-GWEN_LIST_FUNCTION_LIB_DEFS(AB_ACCOUNT_STATUS,
-                            AB_AccountStatus,
-                            AQBANKING_API)
+GWEN_LIST_FUNCTION_DEFS(AB_ACCOUNT_STATUS, AB_AccountStatus)
 
 AB_ACCOUNT_STATUS_LIST *AB_AccountStatus_List_dup(const AB_ACCOUNT_STATUS_LIST *asl);
 
