@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile: jobmultitransfer_l.h,v $
-                             -------------------
-    cvs         : $Id: jobmultitransfer_l.h,v 1.1 2005/08/08 02:24:41 aquamaniac Exp $
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2013 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -15,19 +12,16 @@
 #define AH_JOBMULTITRANSFER_L_H
 
 
-#include <aqhbci/jobmultitransfer.h>
+#include "accountjob_l.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include <gwenhywfar/gwentime.h>
 
 
-int AH_Job_MultiTransferBase_GetTransferCount(AH_JOB *j);
+AH_JOB *AH_Job_MultiTransfer_new(AB_USER *u,
+                                 AB_ACCOUNT *account);
 
-
-#ifdef __cplusplus
-}
-#endif
+AH_JOB *AH_Job_MultiDebitNote_new(AB_USER *cu,
+                                  AB_ACCOUNT *account);
 
 
 #endif /* AH_JOBMULTITRANSFER_L_H */
