@@ -1,9 +1,6 @@
 /***************************************************************************
- $RCSfile: generic.c,v $
-                             -------------------
-    cvs         : $Id: generic.c,v 1.6 2005/08/17 20:10:18 aquamaniac Exp $
     begin       : Mon Mar 01 2004
-    copyright   : (C) 2004 by Martin Preuss
+    copyright   : (C) 2004-2010 by Martin Preuss
     email       : martin@libchipcard.de
 
  ***************************************************************************
@@ -15,11 +12,14 @@
 #endif
 
 #include "generic_p.h"
+#include "i18n_l.h"
 
 #include <aqbanking/banking_be.h>
 #include <gwenhywfar/debug.h>
 #include <gwenhywfar/text.h>
-#include <gwenhywfar/waitcallback.h>
+#include <gwenhywfar/gui.h>
+#include <gwenhywfar/syncio_file.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -40,7 +40,6 @@ GWEN_INHERIT(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC)
 
 
 AB_BANKINFO_PLUGIN *AB_BankInfoPluginGENERIC_new(AB_BANKING *ab,
-                                                 GWEN_DB_NODE *db,
                                                  const char *country){
   AB_BANKINFO_PLUGIN *bip;
   AB_BANKINFO_PLUGIN_GENERIC *bde;
@@ -52,7 +51,6 @@ AB_BANKINFO_PLUGIN *AB_BankInfoPluginGENERIC_new(AB_BANKING *ab,
                        bip, bde, AB_BankInfoPluginGENERIC_FreeData);
 
   bde->banking=ab;
-  bde->dbData=db;
   bde->country=strdup(country);
   AB_BankInfoPlugin_SetGetBankInfoFn(bip, AB_BankInfoPluginGENERIC_GetBankInfo);
   AB_BankInfoPlugin_SetGetBankInfoByTemplateFn(bip,
@@ -63,11 +61,13 @@ AB_BANKINFO_PLUGIN *AB_BankInfoPluginGENERIC_new(AB_BANKING *ab,
 
 
 
-void AB_BankInfoPluginGENERIC_FreeData(void *bp, void *p){
+void GWENHYWFAR_CB AB_BankInfoPluginGENERIC_FreeData(void *bp, void *p){
   AB_BANKINFO_PLUGIN_GENERIC *bde;
 
   bde=(AB_BANKINFO_PLUGIN_GENERIC*)p;
   free(bde->country);
+  if (bde->dataDir) free(bde->dataDir);
+
   GWEN_FREE_OBJECT(bde);
 }
 
@@ -106,7 +106,12 @@ void AB_BankInfoPluginGENERIC__GetDataDir(AB_BANKINFO_PLUGIN *bip,
 
         s=GWEN_StringListEntry_Data(se);
         GWEN_Buffer_AppendString(buf, s);
-        GWEN_Buffer_AppendString(buf, DIRSEP "bankinfo" DIRSEP);
+	GWEN_Buffer_AppendString(buf,
+				 DIRSEP
+				 "aqbanking"
+				 DIRSEP
+				 "bankinfo"
+				 DIRSEP);
         GWEN_Buffer_AppendString(buf, bde->country);
         pos=GWEN_Buffer_GetPos(buf);
         GWEN_Buffer_AppendString(buf, DIRSEP);
@@ -123,6 +128,7 @@ void AB_BankInfoPluginGENERIC__GetDataDir(AB_BANKINFO_PLUGIN *bip,
         GWEN_Buffer_Reset(buf);
         se=GWEN_StringListEntry_Next(se);
       }
+      GWEN_Buffer_free(buf);
     }
     GWEN_StringList_free(sl);
   }
@@ -136,10 +142,10 @@ AB_BANKINFO *AB_BankInfoPluginGENERIC__ReadBankInfo(AB_BANKINFO_PLUGIN *bip,
   AB_BANKINFO_PLUGIN_GENERIC *bde;
   GWEN_BUFFER *pbuf;
   AB_BANKINFO *bi;
-  GWEN_BUFFEREDIO *bio;
-  int fd;
   GWEN_DB_NODE *dbT;
-  GWEN_TYPE_UINT32 pos;
+  GWEN_SYNCIO *sio;
+  uint32_t pos;
+  int rv;
 
   assert(bip);
   bde=GWEN_INHERIT_GETDATA(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC,
@@ -156,43 +162,42 @@ AB_BANKINFO *AB_BankInfoPluginGENERIC__ReadBankInfo(AB_BANKINFO_PLUGIN *bip,
   /* get path */
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/banks.data");
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "banks.data");
 
-  /* open file */
-  fd=open(GWEN_Buffer_GetStart(pbuf), O_RDONLY | O_EXCL);
-  if (fd==-1) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN, "open(%s): %s",
-              GWEN_Buffer_GetStart(pbuf), strerror(errno));
+  sio=GWEN_SyncIo_File_new(GWEN_Buffer_GetStart(pbuf), GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
     GWEN_Buffer_free(pbuf);
-    return 0;
+    return NULL;
   }
+
   /* seek position */
   DBG_VERBOUS(0, "Seeking to %08x (%d)", pos, pos);
-  if ((off_t)-1==lseek(fd, pos, SEEK_SET)) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN, "lseek(%s, "GWEN_TYPE_TMPL_UINT32"): %s",
-              GWEN_Buffer_GetStart(pbuf),
+  if ((int64_t)-1==GWEN_SyncIo_File_Seek(sio, pos, GWEN_SyncIo_File_Whence_Set)) {
+    DBG_ERROR(AQBANKING_LOGDOMAIN,
+	      "seek(%s, %u): %s",
+	      GWEN_Buffer_GetStart(pbuf),
               pos,
-              strerror(errno));
-    close(fd);
-    GWEN_Buffer_free(pbuf);
-    return 0;
+	      strerror(errno));
+    GWEN_SyncIo_Disconnect(sio);
+    GWEN_SyncIo_free(sio);
+    return NULL;
   }
 
-  bio=GWEN_BufferedIO_File_new(fd);
-  GWEN_BufferedIO_SetReadBuffer(bio, 0, 512);
-
   /* read data */
   dbT=GWEN_DB_Group_new("bank");
-
-  if (GWEN_DB_ReadFromStream(dbT, bio,
-                             GWEN_DB_FLAGS_DEFAULT |
-                             GWEN_DB_FLAGS_STOP_ON_EMPTY_LINE |
-                             GWEN_PATH_FLAGS_CREATE_GROUP)) {
-    DBG_ERROR(0, "Could not load file \"%s\"",
-              GWEN_Buffer_GetStart(pbuf));
+  rv=GWEN_DB_ReadFromIo(dbT, sio,
+			GWEN_DB_FLAGS_DEFAULT |
+			GWEN_PATH_FLAGS_CREATE_GROUP|
+			GWEN_DB_FLAGS_UNTIL_EMPTY_LINE);
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
+  if (rv<0) {
+    DBG_INFO(AQBANKING_LOGDOMAIN, "Could not load file \"%s\" (%d)", GWEN_Buffer_GetStart(pbuf), rv);
     GWEN_DB_Group_free(dbT);
-    GWEN_BufferedIO_Abandon(bio);
-    GWEN_BufferedIO_free(bio);
     GWEN_Buffer_free(pbuf);
     return 0;
   }
@@ -200,8 +205,6 @@ AB_BANKINFO *AB_BankInfoPluginGENERIC__ReadBankInfo(AB_BANKINFO_PLUGIN *bip,
   bi=AB_BankInfo_fromDb(dbT);
   assert(bi);
   GWEN_DB_Group_free(dbT);
-  GWEN_BufferedIO_Close(bio);
-  GWEN_BufferedIO_free(bio);
   GWEN_Buffer_free(pbuf);
 
   return bi;
@@ -231,7 +234,7 @@ AB_BANKINFO *AB_BankInfoPluginGENERIC__SearchbyCode(AB_BANKINFO_PLUGIN *bip,
 
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/blz.idx");
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "blz.idx");
   f=fopen(GWEN_Buffer_GetStart(pbuf), "r");
   if (!f) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "fopen(%s): %s",
@@ -285,7 +288,7 @@ int AB_BankInfoPluginGENERIC__AddById(AB_BANKINFO_PLUGIN *bip,
   GWEN_BUFFER *pbuf;
   FILE *f;
   char lbuf[512];
-  GWEN_TYPE_UINT32 count=0;
+  uint32_t count=0;
 
   assert(bip);
   bde=GWEN_INHERIT_GETDATA(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC,
@@ -294,14 +297,14 @@ int AB_BankInfoPluginGENERIC__AddById(AB_BANKINFO_PLUGIN *bip,
 
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/blz.idx");
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "blz.idx");
   f=fopen(GWEN_Buffer_GetStart(pbuf), "r");
   if (!f) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "fopen(%s): %s",
              GWEN_Buffer_GetStart(pbuf),
              strerror(errno));
     GWEN_Buffer_free(pbuf);
-    return AB_ERROR_NOT_AVAILABLE;
+    return GWEN_ERROR_NOT_AVAILABLE;
   }
 
   while(!feof(f)) {
@@ -339,7 +342,7 @@ int AB_BankInfoPluginGENERIC__AddById(AB_BANKINFO_PLUGIN *bip,
   fclose(f);
   if (!count) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "Bank %s not found", bankId);
-    return AB_ERROR_NOT_FOUND;
+    return GWEN_ERROR_NOT_FOUND;
   }
   return 0;
 }
@@ -353,7 +356,7 @@ int AB_BankInfoPluginGENERIC__AddByBic(AB_BANKINFO_PLUGIN *bip,
   GWEN_BUFFER *pbuf;
   FILE *f;
   char lbuf[512];
-  GWEN_TYPE_UINT32 count=0;
+  uint32_t count=0;
 
   assert(bip);
   bde=GWEN_INHERIT_GETDATA(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC,
@@ -362,14 +365,14 @@ int AB_BankInfoPluginGENERIC__AddByBic(AB_BANKINFO_PLUGIN *bip,
 
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/bic.idx");
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "bic.idx");
   f=fopen(GWEN_Buffer_GetStart(pbuf), "r");
   if (!f) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "fopen(%s): %s",
              GWEN_Buffer_GetStart(pbuf),
              strerror(errno));
     GWEN_Buffer_free(pbuf);
-    return AB_ERROR_NOT_AVAILABLE;
+    return GWEN_ERROR_NOT_AVAILABLE;
   }
 
   while(!feof(f)) {
@@ -406,7 +409,7 @@ int AB_BankInfoPluginGENERIC__AddByBic(AB_BANKINFO_PLUGIN *bip,
   fclose(f);
   if (!count) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "Bank %s not found", bic);
-    return AB_ERROR_NOT_FOUND;
+    return GWEN_ERROR_NOT_FOUND;
   }
   return 0;
 }
@@ -421,7 +424,7 @@ int AB_BankInfoPluginGENERIC__AddByNameAndLoc(AB_BANKINFO_PLUGIN *bip,
   GWEN_BUFFER *pbuf;
   FILE *f;
   char lbuf[512];
-  GWEN_TYPE_UINT32 count=0;
+  uint32_t count=0;
 
   assert(bip);
   bde=GWEN_INHERIT_GETDATA(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC,
@@ -435,7 +438,7 @@ int AB_BankInfoPluginGENERIC__AddByNameAndLoc(AB_BANKINFO_PLUGIN *bip,
 
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/namloc.idx");
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "namloc.idx");
   f=fopen(GWEN_Buffer_GetStart(pbuf), "r");
   if (!f) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "fopen(%s): %s",
@@ -443,7 +446,7 @@ int AB_BankInfoPluginGENERIC__AddByNameAndLoc(AB_BANKINFO_PLUGIN *bip,
              strerror(errno));
     GWEN_Buffer_free(pbuf);
     DBG_ERROR(AQBANKING_LOGDOMAIN, "namloc index file not available");
-    return AB_ERROR_NOT_AVAILABLE;
+    return GWEN_ERROR_NOT_AVAILABLE;
   }
 
   while(!feof(f)) {
@@ -488,7 +491,7 @@ int AB_BankInfoPluginGENERIC__AddByNameAndLoc(AB_BANKINFO_PLUGIN *bip,
   fclose(f);
   if (!count) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "Bank %s/%s not found", name, loc);
-    return AB_ERROR_NOT_FOUND;
+    return GWEN_ERROR_NOT_FOUND;
   }
   return 0;
 }
@@ -497,7 +500,7 @@ int AB_BankInfoPluginGENERIC__AddByNameAndLoc(AB_BANKINFO_PLUGIN *bip,
 
 int AB_BankInfoPluginGENERIC__CmpTemplate(AB_BANKINFO *bi,
                                           const AB_BANKINFO *tbi,
-                                          GWEN_TYPE_UINT32 flags) {
+                                          uint32_t flags) {
   const char *s;
   const char *t;
   
@@ -590,13 +593,14 @@ int AB_BankInfoPluginGENERIC__CmpTemplate(AB_BANKINFO *bi,
 int AB_BankInfoPluginGENERIC_AddByTemplate(AB_BANKINFO_PLUGIN *bip,
                                            AB_BANKINFO *tbi,
                                            AB_BANKINFO_LIST2 *bl,
-                                           GWEN_TYPE_UINT32 flags){
+                                           uint32_t flags){
   AB_BANKINFO_PLUGIN_GENERIC *bde;
-  GWEN_TYPE_UINT32 count=0;
-  GWEN_TYPE_UINT32 i=0;
-  GWEN_BUFFEREDIO *bio;
+  uint32_t count=0;
+  uint32_t i=0;
   GWEN_BUFFER *pbuf;
-  int fd;
+  uint32_t progressId;
+  GWEN_SYNCIO *sio;
+  int rv;
 
   assert(bip);
   bde=GWEN_INHERIT_GETDATA(AB_BANKINFO_PLUGIN, AB_BANKINFO_PLUGIN_GENERIC,
@@ -606,51 +610,60 @@ int AB_BankInfoPluginGENERIC_AddByTemplate(AB_BANKINFO_PLUGIN *bip,
   /* get path */
   pbuf=GWEN_Buffer_new(0, 256, 0, 1);
   AB_BankInfoPluginGENERIC__GetDataDir(bip, pbuf);
-  GWEN_Buffer_AppendString(pbuf, "/banks.data");
-  /* open file */
-  fd=open(GWEN_Buffer_GetStart(pbuf), O_RDONLY | O_EXCL);
-  if (fd==-1) {
-    DBG_ERROR(AQBANKING_LOGDOMAIN,
-              "open(%s): %s",
-              GWEN_Buffer_GetStart(pbuf),
-              strerror(errno));
-    GWEN_Buffer_free(pbuf);
-    return AB_ERROR_NOT_FOUND;
-  }
-  bio=GWEN_BufferedIO_File_new(fd);
-  GWEN_BufferedIO_SetReadBuffer(bio, 0, 1024);
+  GWEN_Buffer_AppendString(pbuf, DIRSEP "banks.data");
 
-  while(!GWEN_BufferedIO_CheckEOF(bio)) {
+  /* open file */
+  sio=GWEN_SyncIo_File_new(GWEN_Buffer_GetStart(pbuf), GWEN_SyncIo_File_CreationMode_OpenExisting);
+  GWEN_SyncIo_AddFlags(sio, GWEN_SYNCIO_FILE_FLAGS_READ);
+  rv=GWEN_SyncIo_Connect(sio);
+  if (rv<0) {
+    DBG_INFO(GWEN_LOGDOMAIN, "here (%d)", rv);
+    GWEN_SyncIo_free(sio);
+    GWEN_Buffer_free(pbuf);
+    return rv;
+  }
+
+  progressId=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
+				    GWEN_GUI_PROGRESS_ALLOW_EMBED |
+				    GWEN_GUI_PROGRESS_SHOW_PROGRESS |
+				    GWEN_GUI_PROGRESS_SHOW_ABORT,
+				    I18N("Scanning bank database..."),
+				    NULL,
+				    GWEN_GUI_PROGRESS_NONE,
+				    0);
+
+  for (;;) {
     GWEN_DB_NODE *dbT;
     AB_BANKINFO *bi;
-    int pos;
 
     if (i & ~63) {
-      if (GWEN_WaitCallbackProgress(AB_BANKING_PROGRESS_NONE)==
-          GWEN_WaitCallbackResult_Abort) {
-        DBG_ERROR(AQBANKING_LOGDOMAIN,
-                  "Aborted by user");
-        GWEN_BufferedIO_Abandon(bio);
-        GWEN_BufferedIO_free(bio);
-        GWEN_Buffer_free(pbuf);
-        return AB_ERROR_USER_ABORT;
+      if (GWEN_Gui_ProgressAdvance(progressId, GWEN_GUI_PROGRESS_NONE)==
+	  GWEN_ERROR_USER_ABORTED) {
+	DBG_INFO(GWEN_LOGDOMAIN, "User aborted");
+	GWEN_Gui_ProgressEnd(progressId);
+	DBG_ERROR(AQBANKING_LOGDOMAIN,
+		  "Aborted by user");
+	GWEN_SyncIo_Disconnect(sio);
+	GWEN_SyncIo_free(sio);
+	GWEN_Buffer_free(pbuf);
+	return GWEN_ERROR_USER_ABORTED;
       }
     }
 
     dbT=GWEN_DB_Group_new("bank");
-    pos=GWEN_BufferedIO_GetBytesRead(bio);
-    if (GWEN_DB_ReadFromStream(dbT, bio,
-                               GWEN_DB_FLAGS_DEFAULT |
-                               GWEN_DB_FLAGS_STOP_ON_EMPTY_LINE |
-                               GWEN_PATH_FLAGS_CREATE_GROUP)) {
+    if (GWEN_DB_ReadFromIo(dbT, sio,
+			   GWEN_DB_FLAGS_DEFAULT |
+			   GWEN_PATH_FLAGS_CREATE_GROUP |
+			   GWEN_DB_FLAGS_UNTIL_EMPTY_LINE)) {
       DBG_ERROR(AQBANKING_LOGDOMAIN,
                 "Could not read from file \"%s\"",
                 GWEN_Buffer_GetStart(pbuf));
+      GWEN_Gui_ProgressEnd(progressId);
       GWEN_DB_Group_free(dbT);
-      GWEN_BufferedIO_Abandon(bio);
-      GWEN_BufferedIO_free(bio);
+      GWEN_SyncIo_Disconnect(sio);
+      GWEN_SyncIo_free(sio);
       GWEN_Buffer_free(pbuf);
-      return AB_ERROR_GENERIC;
+      return GWEN_ERROR_GENERIC;
     }
 
     bi=AB_BankInfo_fromDb(dbT);
@@ -666,13 +679,15 @@ int AB_BankInfoPluginGENERIC_AddByTemplate(AB_BANKINFO_PLUGIN *bip,
     i++;
   } /* while */
 
-  GWEN_BufferedIO_Close(bio);
-  GWEN_BufferedIO_free(bio);
+  GWEN_Gui_ProgressEnd(progressId);
+
+  GWEN_SyncIo_Disconnect(sio);
+  GWEN_SyncIo_free(sio);
   GWEN_Buffer_free(pbuf);
 
   if (count==0) {
     DBG_INFO(AQBANKING_LOGDOMAIN, "No matching bank found");
-    return AB_ERROR_NOT_FOUND;
+    return GWEN_ERROR_NOT_FOUND;
   }
 
   return 0;
@@ -684,7 +699,7 @@ int AB_BankInfoPluginGENERIC_SearchbyTemplate(AB_BANKINFO_PLUGIN *bip,
                                               AB_BANKINFO *tbi,
                                               AB_BANKINFO_LIST2 *bl){
   AB_BANKINFO_PLUGIN_GENERIC *bde;
-  GWEN_TYPE_UINT32 flags;
+  uint32_t flags;
   int rv;
   const char *s;
 
@@ -756,9 +771,9 @@ int AB_BankInfoPluginGENERIC_SearchbyTemplate(AB_BANKINFO_PLUGIN *bip,
   else {
     DBG_ERROR(AQBANKING_LOGDOMAIN,
 	      "No quick search implemented for these flags (%08x)", flags);
-    rv=AB_ERROR_NOT_AVAILABLE;
+    rv=GWEN_ERROR_NOT_AVAILABLE;
   }
-  if (rv==AB_ERROR_NOT_AVAILABLE) {
+  if (rv==GWEN_ERROR_NOT_AVAILABLE) {
     rv=AB_BankInfoPluginGENERIC_AddByTemplate(bip, tbi, bl, flags);
   }
 
